import {
  IEventParam,
  Metadata,
  Http,
  Loader,
  Datasource,
} from '@zeedhi/core';
import {
  Select,
  GridEditable,
  Form,
  Switch,
} from '@zeedhi/common';

export class ParametersDashboardController {
  private profileCurrentRow: any = {};
  
  get hasNoProfile() {
    return !this.profileCurrentRow.NRPAINEL;
  }

  get hasProfile() {
    return !!this.profileCurrentRow.NRPAINEL;
  }

  public onChangeCurrentRowBasicUsage({ component }: IEventParam<Datasource>) {
    this.profileCurrentRow = component.filter;
  }
  
  get disabled(): boolean {
    const grid = Metadata.getInstance<GridEditable>('parametersEditGrid');
    return grid.selectedRows.length <= 0;
  }
  
  public deleteProfile({ component }: any) {
    const grid = Metadata.getInstance<GridEditable>('parametersEditGrid');
    const rows = component.parent.selectedRows;
    Http.post('/dellProfileAccess', {
      requestType: 'DataSet',
      dataset: rows,
    });
    grid.datasource.get();
  }
  
  public saveProfileAccess() {
    const formProfile = Metadata.getInstance<Form>('FormAddProfile');
    const perfil = formProfile.getChildInstance<Select>('PERFIL');
    const initialPanel = formProfile.getChildInstance<Switch>('IDPADRAODASH');
    const gridProfile = Metadata.getInstance<GridEditable>('parametersEditGrid');
    if (perfil.value) {
      const conditions = {
        NRPERFILACESSO: perfil.value,
        IDPADRAODASH: initialPanel.value,
        CDFILIAL: perfil.datasource.filter.CDFILIAL,
        NRPAINEL: perfil.datasource.filter.NRPAINEL,
      };
        
      Http.post('/addProfileAccess', {
        requestType: 'Row',
        row: conditions,
      }).then(() => {
        gridProfile.datasource.get();
        const controllerProfile = Loader.getInstance('AddProfileAccessModalController');
        controllerProfile.destroyProfileModal();
      });
    } else {
      formProfile.validate();
    }
  }
  
  public profileGridClick({ row }: IEventParam<GridEditable>) {
    const indicator = Metadata.getInstance<GridEditable>('indicator');
    const conditions = {
      NRPERFACLPAINEL: row.NRPERFACLPAINEL,
      NRPAINEL: row.NRPAINEL,
    };
    indicator.datasource.filter = conditions;
    indicator.datasource.get();
  }
}
