import {
  Metadata,
  Http,
  IEventParam,
} from '@zeedhi/core';
import {
  Select,
  GridEditable,
  Button,
  Number,
  Switch,
  AlertService,
  DialogService,
} from '@zeedhi/common';
import {} from '@zeedhi/teknisa-components-common';

export class ParametersIndicatorsController {
/**
 * Creating the alerts
 *
 * Create the standard alerts for the main uses(success and fail).
 *
 * @access public
 * @author Regis Freitas
  */
  public savedSuccessfully() {
    AlertService.show({
      name: 'saveSucessAlert',
      text: 'Parametrização salva com sucesso.',
      color: 'success',
      timeout: 3000,
      top: true,
      right: true,
    });
  }
  
  public saveFailed() {
    AlertService.show({
      name: 'saveFailedAlert',
      text: 'Verifique os campos inválidos.',
      color: 'error',
      timeout: 5000,
      top: true,
      right: true,
    });
  }
  
  /**
 * Main function to create the confirmation alert
 *
 * A function that receive as a param the event that will happen when clicked the respective button.
 *
 * @param (cancelConfirmation) the event tha will happen
 * @return a object with all properties of the alert modal
 *
 * @access private
 * @author Regis Freitas
  */
  private createConfirmationDialog(cancelConfirmation) {
    return {
      name: 'buttons-dialog',
      type: 'warning',
      title: 'Cancelar Operação?',
      text: 'Todas alterações serão perdidas, deseja continuar a operação?',
      buttons: [
        {
          name: 'no', component: 'ZdButton', label: 'Não', events: this.closeConfirmationModal, outline: true,
        },
        {
          name: 'yes', component: 'ZdButton', label: 'Sim', events: cancelConfirmation,
        },
      ],
    };
  }
  
  public closeConfirmationModal: any = {
    click: () => DialogService.hide(),
  };
  
  /**
 * The 3 methods bellow will work the same way
 *
 * This methods will be linked with the 'yes' button, and when clicked, will call all data
 * from the DB
 *
 * @access public
 * @author Regis Freitas
  */
  public servicesCancelConfirmation = {
    click: async () => {
      const grid = Metadata.getInstance<GridEditable>('servicesGrid');
      await grid.cancelEditedRows();
      this.onInlineEditToggle({ component: grid });
      DialogService.hide();
    },
  };

  public unitManagerCancelConfirmation = {
    click: () => this.unitManagerBuilder(),
  };
  
  public afTrackingCancelConfirmation = {
    click: () => this.aFTrackingBuilder(),
  };
  
  public cancelChangesConfirmationServices() {
    DialogService.show(this.createConfirmationDialog(this.servicesCancelConfirmation));
  }

  public cancelChangesConfirmationUnitManager() {
    DialogService.show(this.createConfirmationDialog(this.unitManagerCancelConfirmation));
  }
  
  public cancelChangesConfirmationAfTracking() {
    DialogService.show(this.createConfirmationDialog(this.afTrackingCancelConfirmation));
  }

  /**
 * This method will return all the services related with the unit that was selected
 *
 * This method retrieves the 'select' field and my grid component, and will search on the DB for the
 * CDFILIAL that was selected on the 'select' field, and then will return all the services related with this CDFILIAL id.
 *
 * @return the grid with all fields populated
 *
 * @access public
 * @author Regis Freitas
  */
  public unitFilter() {
    const unitName = Metadata.getInstance<Select>('unitName');
    const grid = Metadata.getInstance<GridEditable>('servicesGrid');
    const idUnit = unitName.value;
    grid.datasource.filter = idUnit;
    const sendObject = {
      filter: [{
        name: 'CDFILIAL',
        operator: '=',
        value: idUnit,
      }],
    };
    
    // Http.post('/unitServices', {
    //   requestType: 'FilterData',
    //   filter: sendObject,
    // }).then((response) => {
    //   const data = response.data.dataset.unitServices;
    //   grid.datasource.data = data;
    //   grid.datasource.setFilter([]);
    //   grid.datasource.updateData(data);
    // });
    Http.post('/unitServices', sendObject).then((response) => {
      const data = response.data.dataset.unitServices;
      grid.datasource.data = data;
      grid.datasource.setFilter([]);
      grid.datasource.updateData(data);
    });
  }
  
  public showIndicadtorsOnDashboard({ component }: IEventParam<GridEditable>) {
    const gridEditable = component.getEditedRows();
    Http.post('/saveOnDashboard', {
      requestType: 'DataSet',
      dataset: gridEditable,
    });
  }
  
  public isNotInlineToggleEditing = true;
  
  public onInlineEditToggle({ component }: IEventParam<GridEditable>) {
    this.isNotInlineToggleEditing = !component.editing;
  }
  
  public async saveEditingToggle({ component }: IEventParam<Button>) {
    const grid = component.parent as GridEditable;
    this.showIndicadtorsOnDashboard({ component: grid });
    await grid.saveEditedRows();
    this.onInlineEditToggle({ component: grid });
    this.savedSuccessfully();
  }
  
  public buttonStateUnitManager = true;
  
  public activateButtonUnitManager() {
    this.buttonStateUnitManager = false;
  }
  
  /**
 * This 2 functions bellow work the same way
 *
 * This functions will retrieve all the fields from the front-end, so they can be populated later.
 * The function 'AfTrackingFields' work the same way.
 *
 * @return the fields from the front-end
 *
 * @access public
 * @author Regis Freitas
  */
  public unitManagerFieldsFirstTable() {
    const fieldsFirstTable = {
      productGrouping: Metadata.getInstance<Select>('productGrouping'),
      xmlSwitch: Metadata.getInstance<Switch>('xmlSwitch'),
    };
    
    return fieldsFirstTable;
  }

  public unitManagerFieldsSecondTable() {
    const fieldsSecondTable = {
      daysOfStashGoal: Metadata.getInstance<Number>('daysOfStashGoal'),
      daysOfStashMinimumValue: Metadata.getInstance<Number>('daysOfStashMinimumValue'),
      daysOfDelayLimit: Metadata.getInstance<Number>('daysOfDelayLimit'),
      daysOfDelayMaximumValue: Metadata.getInstance<Number>('daysOfDelayMaximumValue'),
      valueAdjustmentLimit: Metadata.getInstance<Number>('valueAdjustmentLimit'),
      valueAdjustmentMaximumValue: Metadata.getInstance<Number>('valueAdjustmentMaximumValue'),
      extraRequestsValueLimit: Metadata.getInstance<Number>('extraRequestsValueLimit'),
      extraRequestsValueMaximumValue: Metadata.getInstance<Number>('extraRequestsValueMaximumValue'),
      consumptionDeviationLimit: Metadata.getInstance<Number>('consumptionDeviationLimit'),
      consumptionDeviationMaximumValue: Metadata.getInstance<Number>('consumptionDeviationMaximumValue'),
      daysToApproveMenuLimit: Metadata.getInstance<Number>('daysToApproveMenuLimit'),
      daysToApproveMenuMaximumValue: Metadata.getInstance<Number>('daysToApproveMenuMaximumValue'),
    };
    
    return fieldsSecondTable;
  }

  /**
 * This methods will get the data from the DB and populate the fields on the front-end
 *
 * This method will use a Http.get to retrieve my DB data with a SELECT(SQL), and then populate all the
 * fields with the respective data, since I have a lot of fields with same 'last name' I created an array
 * with this names of the JSON fields that will be populated, and then I just do a forEach to populate
 * them all, using the index of the array I just create.
 * The method 'AfTrackingBuilder' works the same way.
 *
 * @return all the fields populated with the DB data
 *
 * @access public
 * @author Regis Freitas
  */
  public unitManagerBuilder() {
    Http.get('/unitManagerFirstTable').then((response) => {
      const unitManagerFields = this.unitManagerFieldsFirstTable();
      const data = response.data.dataset.unitManagerFirstTable;
      
      unitManagerFields.productGrouping.value = data[0].CDGRUPPROD;
      unitManagerFields.xmlSwitch.value = data[0].IDCONSXMLCOMFIL;
    });
    
    Http.get('/unitManagerSecondTable').then((response) => {
      const unitManagerFields = this.unitManagerFieldsSecondTable();
      const data = response.data.dataset.unitManagerSecondTable;
      
      unitManagerFields.daysOfStashGoal.value = data[0].VRLIMMETA;
      unitManagerFields.daysOfStashMinimumValue.value = data[0].VRLIMMINIMO;
      
      const fieldMappings = [
        'daysOfDelay', 'valueAdjustment', 'extraRequestsValue',
        'consumptionDeviation', 'daysToApproveMenu',
      ];

      fieldMappings.forEach((fieldName, index) => {
        unitManagerFields[`${fieldName}Limit`].value = data[index + 1].VRLIMMETA;
        unitManagerFields[`${fieldName}MaximumValue`].value = data[index + 1].VRLIMMINIMO;
      });
    });
    this.buttonStateUnitManager = true;
    DialogService.hide();
  }
  
  /**
 * This method will save the data retrivied on the DB, if the data are valid
 *
 * This method will receive an sendObject, and the keys that are a array are receiving the
 * data 'NRINDICADORTF', and the .value, so with this 'NRINDICADORTF' I can check in which
 * field of the table I need to save this data.
 * After that I have my array 'conditions', that I pass the values retrivied from the front-end,
 * so that I can check if they are valid to be saved on the DB.
 * And then I check if is valid, so I can return a 'fail' alert or a 'success'.
 * The method 'saveAfTracking' works the same way.
 *
 * @access public
 * @author Regis Freitas
  */
  public saveUnitManager() {
    const sendObject = {
      productGrouping: Metadata.getInstance<Select>('productGrouping').value,
      xmlSwitch: Metadata.getInstance<Switch>('xmlSwitch').value,
      daysOfStashGoal: [1, Metadata.getInstance<Number>('daysOfStashGoal').value],
      daysOfStashMinimumValue: [1, Metadata.getInstance<Number>('daysOfStashMinimumValue').value],
      daysOfDelayLimit: [3, Metadata.getInstance<Number>('daysOfDelayLimit').value],
      daysOfDelayMaximumValue: [3, Metadata.getInstance<Number>('daysOfDelayMaximumValue').value],
      valueAdjustmentLimit: [4, Metadata.getInstance<Number>('valueAdjustmentLimit').value],
      valueAdjustmentMaximumValue: [4, Metadata.getInstance<Number>('valueAdjustmentMaximumValue').value],
      extraRequestsValueLimit: [5, Metadata.getInstance<Number>('extraRequestsValueLimit').value],
      extraRequestsValueMaximumValue: [5, Metadata.getInstance<Number>('extraRequestsValueMaximumValue').value],
      consumptionDeviationLimit: [19, Metadata.getInstance<Number>('consumptionDeviationLimit').value],
      consumptionDeviationMaximumValue: [19, Metadata.getInstance<Number>('consumptionDeviationMaximumValue').value],
      daysToApproveMenuLimit: [21, Metadata.getInstance<Number>('daysToApproveMenuLimit').value],
      daysToApproveMenuMaximumValue: [21, Metadata.getInstance<Number>('daysToApproveMenuMaximumValue').value],
    };
    
    const conditions = [
      {
        value: sendObject.daysOfStashMinimumValue[1],
        limit: sendObject.daysOfStashGoal[1],
      },
      {
        value: sendObject.daysOfDelayLimit[1],
        limit: sendObject.daysOfDelayMaximumValue[1],
      },
      {
        value: sendObject.valueAdjustmentMaximumValue[1],
        limit: sendObject.valueAdjustmentLimit[1],
      },
      {
        value: sendObject.extraRequestsValueMaximumValue[1],
        limit: sendObject.extraRequestsValueLimit[1],
      },
      {
        value: sendObject.consumptionDeviationMaximumValue[1],
        limit: sendObject.consumptionDeviationLimit[1],
      },
      {
        value: sendObject.daysToApproveMenuMaximumValue[1],
        limit: sendObject.daysToApproveMenuLimit[1],
      },
    ];
    
    const allConditionsPassed = conditions.every((condition) => condition.value >= condition.limit);
    
    if (allConditionsPassed) {
      this.savedSuccessfully();
      this.buttonStateUnitManager = true;
      Http.post('/saveUnitManager', sendObject);
    } else {
      this.saveFailed();
    }
  }
  
  public buttonStateAfTracking = true;
  
  public activateButtonAfTracking() {
    this.buttonStateAfTracking = false;
  }
  
  public aFTrackingFields() {
    const fields = {
      AfCriticalValue: Metadata.getInstance<Number>('AfCriticalValue'),
      AfGoal: Metadata.getInstance<Number>('AfGoal'),
      AfAnnualDeliveryGoal: Metadata.getInstance<Number>('AfAnnualDeliveryGoal'),
    };
    
    return fields;
  }
  
  public aFTrackingBuilder() {
    Http.get('/aFTrackingTable').then((response) => {
      const aFTrackingFields = this.aFTrackingFields();
      const data = response.data.dataset.aFTrackingTable;
      
      aFTrackingFields.AfCriticalValue.value = data[0].VRLIMMETA;
      aFTrackingFields.AfGoal.value = data[0].VRLIMMINIMO;
      aFTrackingFields.AfAnnualDeliveryGoal.value = data[1].VRLIMMETA;
    });
    this.buttonStateAfTracking = true;
    DialogService.hide();
  }
  
  public saveAfTracking() {
    const sendObject = {
      AfCriticalValue: [29, Metadata.getInstance<Number>('AfCriticalValue').value],
      AfGoal: [29, Metadata.getInstance<Number>('AfGoal').value],
      AfAnnualDeliveryGoal: [32, Metadata.getInstance<Number>('AfAnnualDeliveryGoal').value],
      AfEmptyField: [32, null],
    };
    const conditions = [
      {
        value: sendObject.AfGoal[1],
        limit: sendObject.AfCriticalValue[1],
      },
      {
        value: sendObject.AfAnnualDeliveryGoal[1],
        limit: sendObject.AfEmptyField[1],
      },
    ];
    
    const allConditionsPassed = conditions.every((condition) => {
      const isValueInRange = condition.value > 0 && condition.value <= 100;
      const isLimitInRange = condition.limit <= 100 && condition.limit >= 0;
      
      return condition.value >= condition.limit && isValueInRange && isLimitInRange;
    });
    
    if (allConditionsPassed) {
      this.savedSuccessfully();
      this.buttonStateAfTracking = true;
      Http.post('/saveAfTracking', sendObject);
    } else {
      this.saveFailed();
    }
  }
  
  public changeValueTo0({ component }: IEventParam<Number>) {
    if (component.value === undefined) {
      component.value = 0;
    }
  }
  
  public changeValueTo1({ component }: IEventParam<Number>) {
    if (component.value === undefined) {
      component.value = 1;
    }
  }
  
  public resetValidationForUpperField({ component }: IEventParam<Number>) {
    const fieldBellow = component.parent.childrenInstances[1];
    if (component.value <= fieldBellow.value) {
      fieldBellow.resetValidation();
    }
  }
  
  public resetValidationForLowerField({ component }: IEventParam<Number>) {
    const fieldAbove = component.parent.childrenInstances[0];
    if (component.value >= fieldAbove.value) {
      fieldAbove.resetValidation();
    }
  }
}
