import { Metadata } from '@zeedhi/core';
import {
  Modal,
  ModalService,
  Select,
  Switch,
  TextInput,
  Form,
} from '@zeedhi/common';

export class AddProfileAccessModalController {
  private addProfile!: Modal;
    
  public async createProfileModal() {
    this.addProfile = await ModalService.createFromJson('modals/addProfileAccessModal', true);
  }

  public destroyProfileModal() {
    this.addProfile?.destroy();
  }
  
  public async openProfileModal() {
    if (this.addProfile) {
      await this.addProfile.show();
      this.applyDataSource();
    } else {
      this.createProfileModal();
      await this.addProfile.show();
      this.applyDataSource();
    }
  }
  
  public applyDataSource() {
    const formProfile = Metadata.getInstance<Form>('FormAddProfile');
    const formFilter = Metadata.getInstance<Form>('formFilterParamDash');
    
    const filial = formProfile.getChildInstance<TextInput>('FILIAL');
    const painel = formProfile.getChildInstance<TextInput>('PAINEL');
    const perfil = formProfile.getChildInstance<Select>('PERFIL');
    
    filial.value = formFilter.value.CDFILIAL.CDNMFILIAL;
    painel.value = formFilter.value.NRPAINEL.NMPAINEL;
    
    const conditions = {
      CDFILIAL: formFilter.value.CDFILIAL.CDFILIAL,
      NRPAINEL: formFilter.value.NRPAINEL.NRPAINEL,
    };
    
    perfil.datasource.filter = conditions;
    perfil.datasource.get();
  }
  
  public closeModalProfile() {
    this.addProfile.hide();
    this.clearInputsFilter();
  }
  
  public clearInputsFilter() {
    const perfil = Metadata.getInstance<Select>('PERFIL');
    const painelSwitch = Metadata.getInstance<Switch>('IDPADRAODASH');
    perfil.value = '';
    painelSwitch.value = 'N';
  }
}
