import Vue from 'vue';
import Zeedhi from '@zeedhi/vue';
import ZeedhiComponents from '@zeedhi/vuetify';
import TeknisaComponents from '@zeedhi/teknisa-components-vuetify';
import { Icons } from '@zeedhi/common';
import { Messages, Metadata } from '@zeedhi/core';
import { ZdDashboardGrid } from '@zeedhi/zd-dashboard-grid';
import {
  ZdECharts,
  echartsCore,
  echartsCharts,
  echartsComponents,
} from '@zeedhi/zd-echarts';
import controllers from '../controllers';
import config from '../config/config';
import '@zeedhi/common/dist/style.css';
import '@zeedhi/vuetify/dist/zd-style.css';
import vuetify from './vuetify';

echartsCore.use([
  echartsCharts.GaugeChart,
  echartsCharts.PieChart,
  echartsCharts.BarChart,
  echartsCharts.LineChart,
  echartsComponents.GridComponent,
]);

Icons.addIcons({
  iconName: 'mdi-icon-id',
});

Vue.use(Zeedhi, {
  controllers,
  config,
});
Vue.use(ZeedhiComponents);
Vue.use(TeknisaComponents);
Vue.component('ZdECharts', ZdECharts);
Vue.component('ZdDashboardGrid', ZdDashboardGrid);

vuetify.framework.icons.values = Icons.getIcons() as any;

const loadMessages = async () => {
  const messages = await Metadata.get('json/messages', true);
  const wordsEsCl = await Metadata.get('json/words/es_cl', true);
  const wordsEnUs = await Metadata.get('json/words/en_us', true);

  Messages.add({
    'pt-BR': {
      translation: {
        ...messages,
      },
    },
    'es-CL': {
      translation: {
        ...wordsEsCl,
      },
    },
    'en-US': {
      translation: {
        ...wordsEnUs,
      },
    },
  });
};

loadMessages();
